function SetupPlayerLines()
    local function CreateArmyLine(data, armyIndex)
        local group = Group(controls.bgStretch)
        
        group.faction = Bitmap(group)
        if armyIndex ~= 0 then
            group.faction:SetTexture(UIUtil.UIFile(UIUtil.GetFactionIcon(data.faction)))
        else
            group.faction:SetTexture(UIUtil.UIFile('/widgets/faction-icons-alpha_bmp/observer_ico.dds'))
        end
        group.faction.Height:Set(14)
        group.faction.Width:Set(14)
        group.faction:DisableHitTest()
        LayoutHelpers.AtLeftTopIn(group.faction, group)
        
        group.color = Bitmap(group.faction)
        group.color:SetSolidColor(data.color)
        group.color.Depth:Set(function() return group.faction.Depth() - 1 end)
        group.color:DisableHitTest()
        LayoutHelpers.FillParent(group.color, group.faction)
        
	if GetFocusArmy() ~= -1 then
	        group.name = UIUtil.CreateText(group, import('/mods/lazyshare/mimc_diplomacy.lua').mimc_getNewNickname(armyIndex, data.nickname), 12, UIUtil.bodyFont)
	else
		group.name = UIUtil.CreateText(group, data.nickname, 12, UIUtil.bodyFont)
	end


        group.name:DisableHitTest()
        LayoutHelpers.AtLeftIn(group.name, group, 16)
        LayoutHelpers.AtVerticalCenterIn(group.name, group)
        group.name:SetColor('ffffffff')
        
        group.score = UIUtil.CreateText(group, '', 12, UIUtil.bodyFont)
        group.score:DisableHitTest()
        LayoutHelpers.AtRightIn(group.score, group)
        LayoutHelpers.AtVerticalCenterIn(group.score, group)
        group.score:SetColor('ffffffff')
        
        group.name.Right:Set(group.score.Left)
        group.name:SetClipToWidth(true)
        
        group.Height:Set(group.faction.Height)
        group.Width:Set(210)
        
        group.armyID = armyIndex
        
        if SessionIsReplay() then
            group.bg = Bitmap(group)
            group.bg:SetSolidColor('00000000')
            group.bg.Height:Set(group.faction.Height)
            group.bg.Left:Set(group.faction.Right)
            group.bg.Right:Set(group.Right)
            group.bg.Top:Set(group.faction.Top)
            group.bg:DisableHitTest()
            group.bg.Depth:Set(group.Depth)
            group.HandleEvent = function(self, event)
                if event.Type == 'MouseEnter' then
                    group.bg:SetSolidColor('ff777777')
                elseif event.Type == 'MouseExit' then
                    group.bg:SetSolidColor('00000000')
                elseif event.Type == 'ButtonPress' then
                    ConExecute('SetFocusArmy '..tostring(self.armyID-1))
                end
            end
        else
            group:DisableHitTest()
        end
        
        return group
    end
    local index = 1
    for armyIndex, armyData in GetArmiesTable().armiesTable do
        if armyData.civilian or not armyData.showScore then continue end
        if not controls.armyLines then 
            controls.armyLines = {}
        end
        controls.armyLines[index] = CreateArmyLine(armyData, armyIndex)
        index = index + 1
    end
    if SessionIsReplay() then
        observerLine = CreateArmyLine({color = 'ffffffff', nickname = LOC("<LOC score_0003>Observer")}, 0)
        observerLine.name.Top:Set(observerLine.Top)
        observerLine.Height:Set(40)
        observerLine.speedText = UIUtil.CreateText(controls.bgStretch, '', 14, UIUtil.bodyFont)
        observerLine.speedText:SetColor('ff00dbff')
        LayoutHelpers.AtRightIn(observerLine.speedText, observerLine)
        observerLine.speedSlider = IntegerSlider(controls.bgStretch, false, -10, 10, 1, 
            UIUtil.SkinnableFile('/slider02/slider_btn_up.dds'), 
            UIUtil.SkinnableFile('/slider02/slider_btn_over.dds'), 
            UIUtil.SkinnableFile('/slider02/slider_btn_down.dds'), 
            UIUtil.SkinnableFile('/dialogs/options/slider-back_bmp.dds'))
            
        observerLine.speedSlider.Left:Set(function() return observerLine.Left() + 5 end)
        observerLine.speedSlider.Right:Set(function() return observerLine.Right() - 20 end)
        observerLine.speedSlider.Bottom:Set(function() return observerLine.Bottom() - 5 end)
        observerLine.speedSlider._background.Left:Set(observerLine.speedSlider.Left)
        observerLine.speedSlider._background.Right:Set(observerLine.speedSlider.Right)
        observerLine.speedSlider._background.Top:Set(observerLine.speedSlider.Top)
        observerLine.speedSlider._background.Bottom:Set(observerLine.speedSlider.Bottom) 
        observerLine.speedSlider._thumb.Depth:Set(function() return observerLine.Depth() + 5 end)
        Tooltip.AddControlTooltip(observerLine.speedSlider._thumb, 'Lobby_Gen_GameSpeed')
        observerLine.speedSlider._background.Depth:Set(function() return observerLine.speedSlider._thumb.Depth() - 1 end)
        LayoutHelpers.AtVerticalCenterIn(observerLine.speedText, observerLine.speedSlider)
        
        observerLine.speedSlider.OnValueChanged = function(self, newValue)
            observerLine.speedText:SetText(string.format("%+d", math.floor(tostring(newValue))))
        end
        
        observerLine.speedSlider.OnValueSet = function(self, newValue)
            ConExecute("WLD_GameSpeed " .. newValue)
        end
        observerLine.speedSlider:SetValue(gameSpeed)
        controls.armyLines[index] = observerLine
        index = index + 1
    end
end

function _OnBeat()
    if sessionInfo.Options.GameSpeed and sessionInfo.Options.GameSpeed == 'adjustable' then
        controls.time:SetText(string.format("%s (%+d)", GetGameTime(), gameSpeed))
    else
        controls.time:SetText(GetGameTime())
    end
    if sessionInfo.Options.NoRushOption and sessionInfo.Options.NoRushOption != 'Off' then
        if tonumber(sessionInfo.Options.NoRushOption) * 60 > GetGameTimeSeconds() then
            local time = (tonumber(sessionInfo.Options.NoRushOption) * 60) - GetGameTimeSeconds()
            controls.time:SetText(LOCF('%02d:%02d:%02d', math.floor(time / 3600), math.floor(time/60), math.mod(time, 60)))
        end
        if not issuedNoRushWarning and tonumber(sessionInfo.Options.NoRushOption) * 60 == math.floor(GetGameTimeSeconds()) then
            import('/lua/ui/game/announcement.lua').CreateAnnouncement('<LOC score_0001>No Rush Time Elapsed', controls.time)
            issuedNoRushWarning = true
        end
    end
    local armiesInfo = GetArmiesTable().armiesTable
    if currentScores then
        for index, scoreData in currentScores do
            for _, line in controls.armyLines do
                if line.armyID == index then
			--MIMC-Stuff
			if GetFocusArmy() ~= -1 then                
		            	line.name:SetText(import('/mods/lazyshare/mimc_diplomacy.lua').mimc_getNewNickname(index, armiesInfo[index].nickname))
            		end
			--
                    if line.OOG then break end
                    line.score:SetText(scoreData.general.score)
                    if GetFocusArmy() == index then
                        line.name:SetColor('ffff7f00')
                        line.score:SetColor('ffff7f00')
                        line.name:SetFont('Arial Bold', 14)
                        line.score:SetFont('Arial Bold', 14)
                        if scoreData.general.currentcap.count > 0 then
                            SetUnitText(scoreData.general.currentunits.count, scoreData.general.currentcap.count)
                        end
                    else
                        line.name:SetColor('ffffffff')
                        line.score:SetColor('ffffffff')
                        line.name:SetFont(UIUtil.bodyFont, 14)
                        line.score:SetFont(UIUtil.bodyFont, 14)
                    end
                    if armiesInfo[index].outOfGame then
                    	--MIMC-Stuff
            			line.name:SetText(armiesInfo[index].nickname)
            			--
                        line.OOG = true
                        line.faction:SetTexture(UIUtil.UIFile('/game/unit-over/icon-skull_bmp.dds'))
                        line.color:SetSolidColor('ff000000')
                        line.name:SetColor('ffa0a0a0')
                        line.score:SetColor('ffa0a0a0')
                    end
                    break
                end
            end
        end
    end
    if observerLine then
        if GetFocusArmy() == -1 then
            observerLine.name:SetColor('ffff7f00')
            observerLine.name:SetFont('Arial Bold', 14)
        else
            observerLine.name:SetColor('ffffffff')
            observerLine.name:SetFont(UIUtil.bodyFont, 14)
        end
    end
    table.sort(controls.armyLines, function(a,b)
        if a.armyID == 0 or b.armyID == 0 then
            return a.armyID >= b.armyID
        else
            if a.score:GetText() == b.score:GetText() then
                return a.name:GetText() < b.name:GetText()
            else
                return tonumber(a.score:GetText()) > tonumber(b.score:GetText())
            end
        end
    end)
    import(UIUtil.GetLayoutFilename('score')).LayoutArmyLines()
end